import unittest
import logging
from vtb_object_bleach.app import Validator, TypeMismatchError, InvalidEmailFormatError, WeakPasswordError, SecurityThreatDetectedError, ValidationError

# Configure logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

class TestValidator(unittest.TestCase):
    
    def setUp(self):
        """Define the schema for validation."""
        self.schema = {
            "name": "string",
            "email": "string",
            "age": "int",
            "password": "string",
            "date": "date",
            "balance": "float"
        }
        self.validator = Validator(self.schema)
    
    def test_valid_data(self):
        """Test valid data that should pass validation."""
        valid_input = {
            "name": "Alice",
            "email": "alice@example.com",
            "age": 25,
            "password": "StrongPass@123",
            "date": "2024-02-07",
            "balance": 150.75
        }
        try:
            self.validator.validate(valid_input)
            logger.info("✅ Test passed: Valid data successfully validated.")
        except ValidationError as e:
            logger.error(f"❌ Test failed: Unexpected validation error - {e}")
            self.fail(f"Validation failed unexpectedly: {e}")
    
    def test_missing_field(self):
        """Test validation when a required field is missing."""
        invalid_input = {
            "name": "Alice",
            "email": "alice@example.com",
            "age": 25,
            "password": "StrongPass@123",
            "balance": 150.75
        }
        with self.assertRaises(ValidationError):
            logger.info("⚠️ Test case: Missing field 'date'")
            self.validator.validate(invalid_input)
    
    def test_type_mismatch(self):
        """Test validation for incorrect data types."""
        invalid_input = {
            "name": "Alice",
            "email": "alice@example.com",
            "age": "twenty-five",  # Invalid type (string instead of int)
            "password": "StrongPass@123",
            "date": "2024-02-07",
            "balance": 150.75
        }
        with self.assertRaises(TypeMismatchError):
            logger.info("⚠️ Test case: Type mismatch for 'age'")
            self.validator.validate(invalid_input)
    
    def test_invalid_email(self):
        """Test email validation with an incorrect format."""
        invalid_input = {
            "name": "Alice",
            "email": "aliceexample.com",  # Invalid email format
            "age": 25,
            "password": "StrongPass@123",
            "date": "2024-02-07",
            "balance": 150.75
        }
        with self.assertRaises(InvalidEmailFormatError):
            logger.info("⚠️ Test case: Invalid email format detected")
            self.validator.validate(invalid_input)
    
    def test_weak_password(self):
        """Test password validation for weak passwords."""
        invalid_input = {
            "name": "Alice",
            "email": "alice@example.com",
            "age": 25,
            "password": "weakpass",  # Weak password
            "date": "2024-02-07",
            "balance": 150.75
        }
        with self.assertRaises(WeakPasswordError):
            logger.info("⚠️ Test case: Weak password detected")
            self.validator.validate(invalid_input)
    
    def test_sql_injection_detection(self):
        """Test SQL injection payloads."""
        sql_payloads = [
            "'; DROP TABLE users; --",
            "' UNION SELECT username, password FROM users --",
            "' OR '1'='1' --",
            "admin' --",
            "' OR IF(1=1, SLEEP(5), 0) --"
        ]
        for payload in sql_payloads:
            with self.assertRaises(SecurityThreatDetectedError):
                logger.info(f"⚠️ Testing SQL Injection Payload: {payload}")
                user_input = {
                    "name": payload,
                    "email": "test@example.com",
                    "age": 25,
                    "password": "StrongPass@123",
                    "date": "2025-01-01",
                    "balance": 100.50
                }
                self.validator.validate(user_input)
                logger.error(f"❌ Security test failed: {payload} was not detected!")
    
if __name__ == "__main__":
    unittest.main()