import re
import logging
from datetime import datetime

# Configure logging
logging.basicConfig(level=logging.INFO,
                    format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

# Custom Exceptions


class ValidationError(Exception):
    pass


class TypeMismatchError(ValidationError):
    pass


class InvalidEmailFormatError(ValidationError):
    pass


class WeakPasswordError(ValidationError):
    pass


class SecurityThreatDetectedError(ValidationError):
    pass


class Validator:
    def __init__(self, schema):
        self.schema = schema
        self.sql_xss_pattern = re.compile(
            r"(\b(select|insert|delete|update|drop|union|truncate|alter|exec|sleep|benchmark)\b|(--|' OR '| OR |#|/\*|\*/))",
            re.IGNORECASE
        )
        self.email_pattern = re.compile(r"^[\w\.-]+@[\w\.-]+\.\w+$")
        self.password_pattern = re.compile(
            r"^(?=.*[A-Z])(?=.*[a-z])(?=.*\d)(?=.*[@$!%*?&])[A-Za-z\d@$!%*?&]{8,}$")

    def validate(self, obj):
        # Ensure all object keys exist in the schema
        for key in obj.keys():
            if key not in self.schema:
                raise ValidationError(f"Unexpected field: {key}")
            
        for key, expected_type in self.schema.items():
            if key not in obj:
                raise ValidationError(f"Missing required field: {key}")

            value = obj[key]
            self._validate_type(key, value, expected_type)
            self._validate_security(key, value)

            if key.lower() == "email":
                self._validate_email(value)
            elif key.lower() == "password":
                self._validate_password(value)

    def _validate_type(self, key, value, expected_type):
        type_mapping = {
            "string": str,
            "int": int,
            "float": float,
            "date": self._validate_date
        }

        if expected_type not in type_mapping:
            raise ValidationError(
                f"Unsupported type '{expected_type}' for field '{key}'")

        expected_func = type_mapping[expected_type]

        if expected_type == "date":
            if not expected_func(value):
                raise TypeMismatchError(
                    f"Field '{key}' must be a valid date (YYYY-MM-DD)")
        elif not isinstance(value, expected_func):
            raise TypeMismatchError(
                f"Field '{key}' must be of type {expected_type}")

    def _validate_date(self, value):
        try:
            datetime.strptime(value, "%Y-%m-%d")
            return True
        except ValueError:
            return False

    def _validate_security(self, key, value):
        if isinstance(value, str) and self.sql_xss_pattern.search(value):
            logger.warning(
                f"Potential security threat detected in field '{key}': {value}")
            raise SecurityThreatDetectedError(
                f"Potential security threat detected in field '{key}'")

    def _validate_email(self, value):
        if not self.email_pattern.match(value):
            raise InvalidEmailFormatError("Invalid email format")

    def _validate_password(self, value):
        if not self.password_pattern.match(value):
            raise WeakPasswordError(
                "Password must be at least 8 characters long, contain an uppercase letter, a lowercase letter, a number, and a special character")
